import uuid

from datetime import datetime

from sqlalchemy import (
    Boolean,
    Column,
    Date,
    DateTime,
    ForeignKey,
    Index,
    Integer,
    Numeric,
    select,
)
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import Session, relationship

from .meta import Base


class Attendance(Base):
    __tablename__ = "attendances"
    id = Column("id", UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    employee_id = Column("employee_id", UUID(as_uuid=True), ForeignKey("employees.id"))
    date = Column("date", Date, nullable=False)
    attendance_type = Column("attendance_type", Integer)
    amount = Column("amount", Numeric(precision=5, scale=2))
    creation_date = Column("creation_date", DateTime())
    user_id = Column("user_id", UUID(as_uuid=True), ForeignKey("users.id"))
    is_valid = Column("is_valid", Boolean)

    __table_args__ = (
        Index(
            "only_one_valid_attendance",
            employee_id,
            date,
            unique=True,
            postgresql_where=(is_valid == True),  # noqa: E712
        ),
    )
    user = relationship("User", primaryjoin="User.id==Attendance.user_id")

    def __init__(
        self,
        id_=None,
        employee_id=None,
        date=None,
        attendance_type=None,
        amount=None,
        creation_date=None,
        user_id=None,
        is_valid=None,
    ):
        self.id = id_
        self.employee_id = employee_id
        self.date = date
        self.attendance_type = attendance_type
        self.amount = amount if amount is not None else 0
        self.creation_date = creation_date or datetime.utcnow()
        self.user_id = user_id
        self.is_valid = is_valid if is_valid is not None else True

    def create(self, db: Session):
        old = (
            db.execute(
                select(Attendance)
                .filter(Attendance.date == self.date)
                .filter(Attendance.employee_id == self.employee_id)
                .filter(Attendance.is_valid == True)  # noqa: E712
            )
            .scalars()
            .one_or_none()
        )
        if old is None or old.attendance_type != self.attendance_type:
            if old is not None:
                old.is_valid = False
            db.add(self)
