import uuid

from sqlalchemy import Column, Date, ForeignKey, Numeric, UniqueConstraint
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import relationship

from .meta import Base


class ClosingStock(Base):
    __tablename__ = "closing_stocks"
    __table_args__ = (UniqueConstraint("date", "cost_centre_id", "sku_id"),)
    id = Column("id", UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    date = Column("date", Date, nullable=False)
    cost_centre_id = Column(
        "cost_centre_id",
        UUID(as_uuid=True),
        ForeignKey("cost_centres.id"),
        nullable=False,
    )
    sku_id = Column("sku_id", UUID(as_uuid=True), ForeignKey("stock_keeping_units.id"), nullable=False)
    quantity = Column("quantity", Numeric(precision=15, scale=2), nullable=False)

    cost_centre = relationship("CostCentre")
    sku = relationship("StockKeepingUnit")

    def __init__(
        self,
        date_=None,
        cost_centre_id=None,
        sku_id=None,
        quantity=None,
        sku=None,
        id_=None,
    ):
        self.date = date_
        self.cost_centre_id = cost_centre_id
        self.sku_id = sku_id
        self.quantity = quantity
        if sku is not None:
            self.sku = sku
            self.sku_id = sku.id
        self.id = id_
