import uuid

from sqlalchemy import Boolean, Column, Unicode
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import relationship

from .meta import Base


class CostCentre(Base):
    __tablename__ = "cost_centres"

    id = Column("id", UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    name = Column("name", Unicode(255), unique=True)
    is_fixture = Column("is_fixture", Boolean, nullable=False)

    accounts = relationship("AccountBase", back_populates="cost_centre")
    journals = relationship("Journal", back_populates="cost_centre")

    @property
    def __name__(self):
        return self.name

    def __init__(self, name=None, id_=None, is_fixture=False):
        self.name = name
        self.id = id_
        self.is_fixture = is_fixture

    @classmethod
    def cost_centre_purchase(cls):
        return uuid.UUID("7b845f95-dfef-fa4a-897c-f0baf15284a3")

    @classmethod
    def cost_centre_kitchen(cls):
        return uuid.UUID("b2d398ce-e3cc-c542-9feb-5d7783e899df")

    @classmethod
    def cost_centre_overall(cls):
        return uuid.UUID("36f59436-522a-0746-ae94-e0f746bf6c0d")

    @classmethod
    def overall(cls):
        return {
            "id": uuid.UUID("36f59436-522a-0746-ae94-e0f746bf6c0d"),
            "name": "Overall",
        }
