from sqlalchemy import Column, Date, ForeignKey, Integer, Numeric, Unicode, func, select
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import Session, relationship

from .account_base import AccountBase


class Employee(AccountBase):
    __tablename__ = "employees"
    __mapper_args__ = {"polymorphic_identity": "employees"}

    id = Column("id", UUID(as_uuid=True), ForeignKey(AccountBase.id), primary_key=True)
    designation = Column("designation", Unicode(255), nullable=False)
    salary = Column("salary", Integer, nullable=False)
    points = Column("points", Numeric(precision=5, scale=2), nullable=False)
    joining_date = Column("joining_date", Date, nullable=False)
    leaving_date = Column("leaving_date", Date, nullable=True)

    attendances = relationship("Attendance", backref="employee", cascade=None, cascade_backrefs=False)
    fingerprints = relationship("Fingerprint", backref="employee", cascade=None, cascade_backrefs=False)

    def __init__(
        self,
        code=None,
        name=None,
        is_starred=None,
        is_active=None,
        cost_centre_id=None,
        designation=None,
        salary=None,
        points=None,
        joining_date=None,
        leaving_date=None,
    ):
        self.designation = designation
        self.salary = salary
        self.points = points
        self.joining_date = joining_date
        self.leaving_date = leaving_date
        super().__init__(
            code=code,
            name=name,
            type_id=10,
            is_starred=is_starred,
            is_active=is_active,
            is_reconcilable=False,
            cost_centre_id=cost_centre_id,
        )

    def create(self, db: Session):
        self.code = db.execute(
            select(func.coalesce(func.max(AccountBase.code), 0) + 1).filter(AccountBase.type_id == self.type_id)
        ).scalar_one()
        self.name += f" ({str(self.code)})"
        db.add(self)
        return self

    def can_delete(self, advanced_delete):
        return super(Employee, self).can_delete(advanced_delete)
