import uuid

from sqlalchemy import Column, ForeignKey, Integer
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import backref, relationship

from .journal import Journal
from .meta import Base


class EmployeeBenefit(Base):
    __tablename__ = "employee_benefit"
    id = Column("id", UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    voucher_id = Column("voucher_id", UUID(as_uuid=True), ForeignKey("vouchers.id"), nullable=False)
    journal_id = Column("journal_id", UUID(as_uuid=True), ForeignKey("journals.id"), nullable=False)
    gross_salary = Column("gross_salary", Integer)
    days_worked = Column("days_worked", Integer)
    esi_ee = Column("esi_employee", Integer)
    pf_ee = Column("pf_employee", Integer)
    esi_er = Column("esi_employer", Integer)
    pf_er = Column("pf_employer", Integer)

    journal = relationship(
        Journal,
        backref=backref("employee_benefit", uselist=False),
        cascade=None,
        cascade_backrefs=False,
    )

    def __init__(
        self,
        id_=None,
        voucher_id=None,
        journal_id=None,
        journal=None,
        gross_salary=None,
        days_worked=None,
        esi_ee=None,
        pf_ee=None,
        esi_er=None,
        pf_er=None,
    ):
        self.id = id_
        self.voucher_id = voucher_id
        self.gross_salary = gross_salary
        self.days_worked = days_worked
        self.esi_ee = esi_ee
        self.pf_ee = pf_ee
        self.esi_er = esi_er
        self.pf_er = pf_er
        if journal is None:
            self.journal_id = journal_id
        else:
            self.journal = journal
