import uuid

from sqlalchemy import Column, ForeignKey, Numeric
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import backref, relationship

from .journal import Journal
from .meta import Base


class Incentive(Base):
    __tablename__ = "incentives"
    id = Column("id", UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    voucher_id = Column("voucher_id", UUID(as_uuid=True), ForeignKey("vouchers.id"), nullable=False)
    journal_id = Column("journal_id", UUID(as_uuid=True), ForeignKey("journals.id"), nullable=False)
    days_worked = Column("days_worked", Numeric(precision=5, scale=1), nullable=False)
    points = Column("points", Numeric(precision=5, scale=2), nullable=False)

    journal = relationship(
        Journal,
        backref=backref("incentive", uselist=False),
        cascade=None,
        cascade_backrefs=False,
    )

    def __init__(
        self,
        id_=None,
        voucher_id=None,
        journal_id=None,
        journal=None,
        days_worked=None,
        points=None,
    ):
        self.id = id_
        self.voucher_id = voucher_id
        if journal is None:
            self.journal_id = journal_id
        else:
            self.journal = journal
        self.days_worked = days_worked
        self.points = points
