import uuid

from sqlalchemy import Column, ForeignKey, Numeric, UniqueConstraint
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.ext.hybrid import hybrid_property
from sqlalchemy.orm import relationship

from .meta import Base


class Inventory(Base):
    __tablename__ = "inventories"
    __table_args__ = (UniqueConstraint("voucher_id", "batch_id"),)
    id = Column("id", UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    voucher_id = Column(
        "voucher_id",
        UUID(as_uuid=True),
        ForeignKey("vouchers.id"),
        nullable=False,
        index=True,
    )
    batch_id = Column("batch_id", UUID(as_uuid=True), ForeignKey("batches.id"), nullable=False)
    quantity = Column("quantity", Numeric(precision=15, scale=2), nullable=False)
    rate = Column("rate", Numeric(precision=15, scale=2), nullable=False)
    tax = Column("tax", Numeric(precision=15, scale=5), nullable=False)
    discount = Column("discount", Numeric(precision=15, scale=5), nullable=False)

    voucher = relationship("Voucher", back_populates="inventories")
    batch = relationship("Batch", back_populates="inventories")

    def __init__(
        self,
        id_=None,
        voucher_id=None,
        batch_id=None,
        quantity=None,
        rate=None,
        tax=None,
        discount=None,
        batch=None,
    ):
        self.id = id_
        self.voucher_id = voucher_id
        if batch is None:
            self.batch_id = batch_id
        else:
            self.batch = batch
        self.quantity = quantity
        self.rate = rate
        self.tax = tax
        self.discount = discount

    @hybrid_property
    def amount(self):
        return self.quantity * self.rate * (1 + self.tax) * (1 - self.discount)

    @amount.expression  # type: ignore[no-redef]
    def amount(cls):
        return cls.quantity * cls.rate * (1 + cls.tax) * (1 - cls.discount)
