import uuid

from sqlalchemy import Column, ForeignKey, Integer, Numeric
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.ext.hybrid import hybrid_property
from sqlalchemy.orm import relationship

from .meta import Base


class Journal(Base):
    __tablename__ = "journals"

    id = Column("id", UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    debit = Column("debit", Integer, nullable=False)
    amount = Column("amount", Numeric(precision=15, scale=2), nullable=False)
    voucher_id = Column(
        "voucher_id",
        UUID(as_uuid=True),
        ForeignKey("vouchers.id"),
        nullable=False,
        index=True,
    )
    account_id = Column("account_id", UUID(as_uuid=True), ForeignKey("accounts.id"), nullable=False)
    cost_centre_id = Column(
        "cost_centre_id",
        UUID(as_uuid=True),
        ForeignKey("cost_centres.id"),
        nullable=False,
    )

    voucher = relationship("Voucher", back_populates="journals")
    account = relationship("AccountBase", back_populates="journals")
    cost_centre = relationship("CostCentre", back_populates="journals")

    @hybrid_property
    def signed_amount(self):
        return self.debit * self.amount

    @property
    def __name__(self):
        return self.name

    def __init__(
        self,
        id_=None,
        debit=None,
        amount=None,
        voucher_id=None,
        account_id=None,
        cost_centre_id=None,
    ):
        self.id = id_
        self.debit = debit
        self.amount = amount
        self.voucher_id = voucher_id
        self.account_id = account_id
        self.cost_centre_id = cost_centre_id
