from __future__ import annotations

import uuid

from datetime import datetime

from sqlalchemy import Column, DateTime, UniqueConstraint
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import relationship
from sqlalchemy.schema import ForeignKey

from .meta import Base


class LoginHistory(Base):
    __tablename__ = "login_history"
    __table_args__ = (UniqueConstraint("user_id", "client_id", "date"),)
    id = Column("login_history_id", UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    user_id = Column("user_id", UUID(as_uuid=True), ForeignKey("users.id"), nullable=False)
    client_id = Column(
        "client_id",
        UUID(as_uuid=True),
        ForeignKey("clients.client_id"),
        nullable=False,
    )
    date = Column("date", DateTime(), nullable=False)

    user = relationship("User", back_populates="login_history")

    def __init__(self, user_id=None, client_id=None, date=None, id_=None):
        self.user_id = user_id
        self.client_id = client_id
        self.date = datetime.utcnow() if date is None else date
        self.id = id_
