import uuid

from sqlalchemy import Boolean, Column, ForeignKey, Integer, Unicode, desc, func, select
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import Session, relationship

from .meta import Base
from .stock_keeping_unit import StockKeepingUnit


class Product(Base):
    __tablename__ = "products"

    id = Column("id", UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    code = Column("code", Integer, unique=True)
    name = Column("name", Unicode(255), nullable=False, unique=True)
    product_group_id = Column(
        "product_group_id",
        UUID(as_uuid=True),
        ForeignKey("product_groups.id"),
        nullable=False,
    )
    account_id = Column("account_id", UUID(as_uuid=True), ForeignKey("accounts.id"), nullable=False)
    is_active = Column("is_active", Boolean, nullable=False)
    is_fixture = Column("is_fixture", Boolean, nullable=False)
    is_purchased = Column("is_purchased", Boolean, nullable=False)
    is_sold = Column("is_sold", Boolean, nullable=False)

    skus = relationship("StockKeepingUnit", back_populates="product", order_by=desc(StockKeepingUnit.is_default))
    product_group = relationship("ProductGroup", back_populates="products")
    recipes = relationship("Recipe", back_populates="product")
    account = relationship("Account", primaryjoin="Account.id==Product.account_id", back_populates="products")

    def __init__(
        self,
        code=None,
        name=None,
        product_group_id=None,
        account_id=None,
        is_active=None,
        is_purchased=None,
        is_sold=None,
        id_=None,
        is_fixture=False,
    ):
        self.code = code
        self.name = name
        self.product_group_id = product_group_id
        self.account_id = account_id
        self.is_active = is_active
        self.is_purchased = is_purchased
        self.is_sold = is_sold
        self.id = id_
        self.is_fixture = is_fixture

    def create(self, db: Session):
        self.code = db.execute(select(func.coalesce(func.max(Product.code), 0) + 1)).scalar_one()
        db.add(self)
        return self

    @classmethod
    def suspense(cls):
        return uuid.UUID("aa79a643-9ddc-4790-ac7f-a41f9efb4c15")
