import uuid

from datetime import datetime

from sqlalchemy import Column, Date, DateTime, ForeignKey, Unicode
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import relationship

from .meta import Base


class RateContract(Base):
    __tablename__ = "rate_contracts"

    id = Column("id", UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    date = Column("date", Date, nullable=False, index=True)
    vendor_id = Column("vendor_id", UUID(as_uuid=True), ForeignKey("accounts.id"), nullable=False)
    valid_from = Column("valid_from", Date(), nullable=True)
    valid_till = Column("valid_till", Date(), nullable=True)
    narration = Column("narration", Unicode(1000), nullable=False)

    user_id = Column("user_id", UUID(as_uuid=True), ForeignKey("users.id"), nullable=False)
    creation_date = Column("creation_date", DateTime(), nullable=False)
    last_edit_date = Column("last_edit_date", DateTime(), nullable=False)

    user = relationship("User")
    vendor = relationship("AccountBase", back_populates="rate_contracts")
    items = relationship("RateContractItem", back_populates="rate_contract")

    def __init__(
        self,
        date=None,
        vendor_id=None,
        valid_from=None,
        valid_till=None,
        narration="",
        user_id=None,
        creation_date=None,
        last_edit_date=None,
        id_=None,
    ):
        self.date = date
        self.vendor_id = vendor_id
        self.valid_from = valid_from
        self.valid_till = valid_till
        self.narration = narration
        self.user_id = user_id
        self.creation_date = creation_date or datetime.utcnow()
        self.last_edit_date = last_edit_date or datetime.utcnow()
        self.id = id_
