import uuid

from sqlalchemy import Column, ForeignKey, Numeric, UniqueConstraint
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import relationship

from .meta import Base


class RateContractItem(Base):
    __tablename__ = "rate_contract_items"
    __table_args__ = (UniqueConstraint("rate_contract_id", "sku_id"),)

    id = Column("id", UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    rate_contract_id = Column(
        "rate_contract_id",
        UUID(as_uuid=True),
        ForeignKey("rate_contracts.id"),
        nullable=False,
        index=True,
    )
    sku_id = Column("sku_id", UUID(as_uuid=True), ForeignKey("stock_keeping_units.id"), nullable=False)
    price = Column("cost_price", Numeric(precision=15, scale=2), nullable=False)

    rate_contract = relationship("RateContract", back_populates="items")
    sku = relationship("StockKeepingUnit")

    def __init__(
        self,
        rate_contract_id=None,
        sku_id=None,
        price=None,
        id_=None,
    ):
        self.rate_contract_id = rate_contract_id
        self.sku_id = sku_id
        self.price = price
        self.id = id_
