import uuid

from datetime import date

from sqlalchemy import Column, Date, ForeignKey, Numeric, Unicode
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import relationship

from .meta import Base


class Recipe(Base):
    __tablename__ = "recipes"

    id = Column("id", UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    product_id = Column("product_id", UUID(as_uuid=True), ForeignKey("products.id"), nullable=False)

    quantity = Column("quantity", Numeric(precision=15, scale=2), nullable=False)
    cost_price = Column("cost_price", Numeric(precision=15, scale=2), nullable=False)
    sale_price = Column("sale_price", Numeric(precision=15, scale=2), nullable=False)
    notes = Column("notes", Unicode(255))

    valid_from = Column("valid_from", Date, nullable=True)
    valid_till = Column("valid_till", Date, nullable=True)

    effective_from = Column("effective_from", Date, nullable=False)
    effective_to = Column("effective_to", Date)

    recipe_items = relationship("RecipeItem", backref="recipe")
    product = relationship("Product", back_populates="recipes")

    def __init__(
        self,
        product_id=None,
        quantity=None,
        cost_price=None,
        sale_price=None,
        valid_from=None,
        valid_till=None,
        notes=None,
        effective_from=None,
        id_=None,
    ):
        self.product_id = product_id
        self.quantity = quantity
        self.cost_price = cost_price
        self.sale_price = sale_price
        self.valid_from = valid_from
        self.valid_till = valid_till
        self.notes = "" if notes is None else notes
        self.effective_from = date.today() if effective_from is None else effective_from
        self.effective_to = None
        self.id = id_
