import uuid

from sqlalchemy import (
    Boolean,
    Column,
    ForeignKey,
    Index,
    Numeric,
    Unicode,
    UniqueConstraint,
    text,
)
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import relationship

from .meta import Base


class StockKeepingUnit(Base):
    __tablename__ = "stock_keeping_units"
    __table_args__ = (
        UniqueConstraint("product_id", "units"),
        Index(
            "unique_true_is_default",
            "product_id",
            unique=True,
            postgresql_where=text("is_default = true"),
        ),
    )

    id = Column("id", UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    product_id = Column("product_id", UUID(as_uuid=True), ForeignKey("products.id"), nullable=False)
    is_default = Column("is_default", Boolean, nullable=False)
    units = Column("units", Unicode(255), nullable=False)
    fraction = Column("fraction", Numeric(precision=15, scale=5), nullable=False)
    fraction_units = Column("fraction_units", Unicode(255), nullable=False)
    product_yield = Column("product_yield", Numeric(precision=15, scale=5), nullable=False)
    price = Column("cost_price", Numeric(precision=15, scale=2), nullable=False)
    sale_price = Column("sale_price", Numeric(precision=15, scale=2), nullable=False)

    product = relationship("Product", back_populates="skus")
    batches = relationship("Batch", back_populates="sku")

    def __init__(
        self,
        product_id=None,
        is_default=None,
        units=None,
        fraction=None,
        fraction_units=None,
        product_yield=None,
        price=None,
        sale_price=None,
        id_=None,
        product=None,
    ):
        if product_id is not None:
            self.product_id = product_id
        self.is_default = is_default
        self.units = units
        self.fraction = fraction
        self.fraction_units = fraction_units
        self.product_yield = product_yield
        self.price = price
        self.sale_price = sale_price
        self.id = id_
        if product is not None:
            self.product = product
