import uuid

from datetime import datetime

from sqlalchemy import Boolean, Column, Date, DateTime, Enum, ForeignKey, Unicode
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import relationship

from .meta import Base
from .voucher_type import VoucherType


class Voucher(Base):
    __tablename__ = "vouchers"

    id = Column("id", UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    date = Column("date", Date, nullable=False, index=True)
    narration = Column("narration", Unicode(1000), nullable=False)
    is_reconciled = Column("is_reconciled", Boolean, nullable=False)
    reconcile_date = Column("reconcile_date", Date, nullable=False)
    is_starred = Column("is_starred", Boolean, nullable=False)
    creation_date = Column("creation_date", DateTime(), nullable=False)
    last_edit_date = Column("last_edit_date", DateTime(), nullable=False)
    voucher_type = Column("voucher_type", Enum(VoucherType), nullable=False)
    user_id = Column("user_id", UUID(as_uuid=True), ForeignKey("users.id"), nullable=False)
    posted = Column("is_posted", Boolean, nullable=False)
    poster_id = Column("poster_id", UUID(as_uuid=True), ForeignKey("users.id"))

    user = relationship("User", primaryjoin="User.id==Voucher.user_id", cascade=None)
    poster = relationship("User", primaryjoin="User.id==Voucher.poster_id", cascade=None)

    journals = relationship(
        "Journal",
        back_populates="voucher",
        cascade="delete, delete-orphan",
        cascade_backrefs=False,
    )

    inventories = relationship(
        "Inventory",
        back_populates="voucher",
        cascade="delete, delete-orphan",
        cascade_backrefs=False,
    )
    employee_benefits = relationship(
        "EmployeeBenefit",
        backref="voucher",
        cascade="delete, delete-orphan",
        cascade_backrefs=False,
    )
    incentives = relationship(
        "Incentive",
        backref="voucher",
        cascade="delete, delete-orphan",
        cascade_backrefs=False,
    )

    def __init__(
        self,
        date=None,
        is_reconciled=False,
        reconcile_date=None,
        is_starred=None,
        narration="",
        posted=False,
        creation_date=None,
        last_edit_date=None,
        voucher_type=None,
        user_id=None,
        poster_id=None,
    ):
        self.date = date
        self.is_reconciled = is_reconciled
        self.reconcile_date = reconcile_date or date
        self.is_starred = is_starred if is_starred is not None else False
        self.narration = narration
        self.posted = posted
        self.creation_date = creation_date or datetime.utcnow()
        self.last_edit_date = last_edit_date or datetime.utcnow()
        self.voucher_type = voucher_type
        self.user_id = user_id
        self.poster_id = poster_id
