import datetime

from fastapi import APIRouter, Depends

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionFuture
from ..models.batch import Batch
from ..schemas.user import UserToken


router = APIRouter()


@router.get("")
def batch_term(
    q: str,
    d: str = None,
    current_user: UserToken = Depends(get_user),
):
    date = None if not d else datetime.datetime.strptime(d, "%d-%b-%Y")
    list_ = []
    with SessionFuture() as db:
        for item in Batch.list(q, include_nil=False, date_=date, db=db):
            text = (
                f"{item.sku.product.name} ({item.sku.units}) {item.quantity_remaining:.2f}@"
                f"{item.rate:.2f} from {item.name.strftime('%d-%b-%Y')}"
            )
            list_.append(
                {
                    "id": item.id,
                    "name": text,
                    "quantityRemaining": round(item.quantity_remaining, 2),
                    "rate": round(item.rate, 2),
                    "tax": round(item.tax, 5),
                    "discount": round(item.discount, 5),
                    "sku": {
                        "id": item.sku.id,
                        "name": f"{item.sku.product.name} ({item.sku.units})",
                    },
                }
            )
        return list_
