from fastapi import APIRouter, Security
from sqlalchemy import delete, desc, distinct, func, over, select
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionFuture
from ..models.attendance import Attendance
from ..schemas.user import UserToken


router = APIRouter()


@router.post("")
def post_check_db(user: UserToken = Security(get_user)):
    info = {}

    with SessionFuture() as db:
        duplicate_attendances = get_duplicate_attendances(db)
        if duplicate_attendances > 0:
            fix_duplicate_attendances(db)
            info["attendanceCount"] = duplicate_attendances
        db.commit()
        return info


def get_duplicate_attendances(db: Session) -> int:
    sub_query = select(
        over(
            distinct(func.first_value(Attendance.id)),
            partition_by=[Attendance.employee_id, Attendance.date],
        )
    ).where(
        Attendance.is_valid == True  # noqa: E712
    )
    query = db.execute(
        select(func.count(Attendance.id)).where(
            ~Attendance.id.in_(sub_query), Attendance.is_valid == True  # noqa: E712
        )
    )
    return query.scalar()


def fix_duplicate_attendances(db: Session) -> None:
    sub = (
        select(
            over(
                distinct(func.first_value(Attendance.id)),
                partition_by=[Attendance.employee_id, Attendance.date],
                order_by=desc(Attendance.creation_date),
            )
        )
        .where(Attendance.is_valid == True)  # noqa: E712
        .subquery()
    )
    db.execute(delete(Attendance).where(~Attendance.id.in_(sub), Attendance.is_valid == True))  # noqa: E712
