from datetime import datetime

from fastapi import APIRouter, Security
from sqlalchemy import select
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionFuture
from ..models.voucher import Voucher
from ..models.voucher_type import VoucherType
from ..schemas.user import UserToken


router = APIRouter()


@router.get("/{date}")
def grid_date(
    date: str,
    user: UserToken = Security(get_user, scopes=["issue"]),
):
    date = datetime.strptime(date, "%d-%b-%Y")
    with SessionFuture() as db:
        return get_grid(date, db)


def get_grid(date, db: Session):
    list_ = []
    query = (
        db.execute(
            select(Voucher)
            .join(Voucher.journals)
            .where(Voucher.date == date, Voucher.voucher_type == VoucherType.ISSUE)
            .order_by(Voucher.creation_date)
        )
        .unique()
        .scalars()
        .all()
    )

    for voucher in query:
        amount = voucher.journals[0].amount
        list_.append(
            {
                "id": str(voucher.id),
                "amount": amount,
                "source": [j.cost_centre.name for j in voucher.journals if j.debit == -1],
                "destination": [j.cost_centre.name for j in voucher.journals if j.debit == 1],
            }
        )
    return list_
