from fastapi import APIRouter, Security
from sqlalchemy import select
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionFuture
from ..models.db_setting import DbSetting
from ..models.user import User
from ..schemas.settings import Maintenance
from ..schemas.user import UserToken


router = APIRouter()


@router.get("", response_model=Maintenance)
def get_maintenance(
    user: UserToken = Security(get_user),
):
    with SessionFuture() as db:
        data = db.execute(select(DbSetting).where(DbSetting.name == "Maintenance")).scalars().one_or_none()
        return info(data, db)


@router.post("", response_model=Maintenance)
def set_maintenance(
    data: Maintenance,
    user: UserToken = Security(get_user, scopes=["maintenance"]),
):
    with SessionFuture() as db:
        maintenance = db.execute(select(DbSetting).where(DbSetting.name == "Maintenance")).scalars().one_or_none()
        if data.enabled is False and maintenance is not None:
            db.delete(maintenance)
            maintenance = None
        elif data.enabled is True and maintenance is None:
            maintenance = DbSetting(name="Maintenance", data=user.id_)
            db.add(maintenance)
        elif data.enabled is True and maintenance.data != user.id_:
            maintenance.data = user.id_
        db.commit()
        return info(maintenance, db)


def info(data, db: Session):
    if data is None:
        return {"enabled": False, "user": ""}
    user = db.execute(select(User).where(User.id == data.data)).scalar_one()
    return {"enabled": True, "user": user.name}
