import uuid

from datetime import date, datetime
from typing import List, Optional

from pydantic import BaseModel, validator

from . import to_camel
from .account import AccountLink
from .rate_contract_item import RateContractItem
from .user_link import UserLink


class RateContractIn(BaseModel):
    date_: date
    vendor: Optional[AccountLink]
    valid_from: date
    valid_till: date
    narration: str
    items: List[RateContractItem]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {
            date: lambda v: v.strftime("%d-%b-%Y"),
            datetime: lambda v: v.strftime("%d-%b-%Y %H:%I"),
        }

    @validator("date_", pre=True)
    def parse_date(cls, value):
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @validator("valid_from", pre=True)
    def parse_creation_date(cls, value):
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y")

    @validator("valid_till", pre=True)
    def parse_last_edit_date(cls, value):
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y")


class RateContract(RateContractIn):
    id_: uuid.UUID
    creation_date: Optional[datetime]
    last_edit_date: Optional[datetime]
    user: Optional[UserLink]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {
            date: lambda v: v.strftime("%d-%b-%Y"),
            datetime: lambda v: v.strftime("%d-%b-%Y %H:%I"),
        }

    # @validator("date_", pre=True)
    # def parse_date(cls, value):
    #     if isinstance(value, date):
    #         return value
    #     return datetime.strptime(value, "%d-%b-%Y").date()
    #
    # @validator("valid_from", pre=True)
    # def parse_creation_date(cls, value):
    #     if isinstance(value, datetime):
    #         return value
    #     return datetime.strptime(value, "%d-%b-%Y %H:%M")
    #
    # @validator("valid_till", pre=True)
    # def parse_last_edit_date(cls, value):
    #     if isinstance(value, datetime):
    #         return value
    #     return datetime.strptime(value, "%d-%b-%Y %H:%M")

    @validator("creation_date", pre=True)
    def parse_creation_date(cls, value):
        if isinstance(value, datetime):
            return value
        return datetime.strptime(value, "%d-%b-%Y %H:%M")

    @validator("last_edit_date", pre=True)
    def parse_last_edit_date(cls, value):
        if isinstance(value, datetime):
            return value
        return datetime.strptime(value, "%d-%b-%Y %H:%M")
