import uuid

from datetime import date
from decimal import Decimal
from typing import List, Optional

from pydantic import BaseModel, Field

from . import to_camel


class ProductLink(BaseModel):
    id_: uuid.UUID = Field(...)
    name: Optional[str]
    units: Optional[str]
    sale_price: Optional[Decimal]
    is_sold: Optional[bool]
    fraction_units: Optional[str]
    fraction: Optional[Decimal]
    product_yield: Optional[Decimal]

    class Config:
        alias_generator = to_camel


class RecipeItem(BaseModel):
    id_: Optional[uuid.UUID]
    product: ProductLink
    quantity: int
    price: int

    class Config:
        fields = {"id_": "id"}


class RecipeIn(BaseModel):
    product: ProductLink

    quantity: Decimal
    cost_price: Decimal
    sale_price: Decimal
    notes: str

    valid_from: Optional[date]
    valid_till: Optional[date]

    effective_from: Optional[date]
    effective_till: Optional[date]

    items: List[RecipeItem]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {date: lambda v: v.strftime("%d-%b-%Y") if v is not None else None}


class Recipe(RecipeIn):
    id_: uuid.UUID

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {date: lambda v: v.strftime("%d-%b-%Y")}


class RecipeBlank(RecipeIn):
    product: Optional[ProductLink]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
